/**
 * Create folder structure
 *
 * @category  WeBuilder Plugin
 * @package   Create Folder Structure
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2016 Peter Klein
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.1
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     45   function ShowModal()
 *    237   function Split(str, splitChar)
 *    255   function SelectfolderButton()
 *    266   function SelectFolder(Sender)
 *    278   function CreateStructure()
 *    335   function UpdateLayouts(Sender)
 *    348   function UpdateProjects(Sender)
 *    362   function SelectProjectPath(Sender)
 *    374   function DeleteLayout()
 *    391   function SaveLayout()
 *    416   function SelectLayout()
 *    448   function OnResize(Sender)
 *    463   function OnExit()
 *    477   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 14
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

var projects = new TIniFile(Regexreplace(Script.GetPath, "plugins.*", "settings", false) + "\\projects.ini");
var layouts = new TIniFile(Script.GetPath + "layouts.ini");
var settings = new TIniFile(Script.GetPath + "settings.ini");
var form, grpOptions, grpEdit, cmbLayout, txtPath, mmEdit, btnDelete, btnSave, currentFolder;

/**
 * Show plugin GUI
 *
 * @return void]
 */
function ShowModal() {

    form = new TForm(WeBuilder);

    var mul = Round(form.PixelsPerInch / 120),
        mleft = 14*mul,
        mtop = 14*mul,
        vspace = 32*mul,
        hdiv = 10*mul,
        topPos = 0,
        leftPos = 0;
        margin = 6*mul;

    form.Width = 540*mul;
    form.Height = 480*mul;
    form.Position = poScreenCenter;
    form.BorderIcons = biSystemMenu; //remove maximize & minimize buttons
    form.Color = clBtnFace;
    form.Font.Color = clWindowText;
    form.Font.Height = -13;
    form.Caption = "Create Folder Structure";
    form.OnResize = "OnResize";

    var groupWidth = form.Width - margin - margin;

  	grpOptions = new TGroupBox(form);
	grpOptions.Parent = form;
    grpOptions.Caption 	= "Folder Structure Layouts";
    grpOptions.Anchors = akLeft + akTop + akRight;
    grpOptions.Width = groupWidth;
    grpOptions.Left = margin;
    grpOptions.Top = margin;

    topPos = mtop + margin;
    leftPos = mleft;

    var lblPath = new TLabel(form);
    lblPath.Parent = grpOptions;
    lblPath.Caption = "Path:";
    lblPath.SetBounds(leftPos, topPos + 7*mul, 60*mul, 16*mul);
    lblPath.Anchors = akLeft + akTop;
    lblPath.ShowHint = true;
    lblPath.Hint = "Enter path to where folder structure should be created. Doubleclick for Folder Dialog.";
    lblPath.Alignment = taRightJustify;

    leftPos += lblPath.Width + 2*mul;

    txtPath = new TEdit(form);
    txtPath.Parent = grpOptions;
    txtPath.SetBounds( leftPos, topPos + 4*mul, groupWidth - mleft*2 - lblPath.Width -28*mul, 24*mul);
    txtPath.Anchors = akLeft + akTop + akRight;
    txtPath.ReadOnly = (Script.ReadSetting("Path field is ReadOnly", "1")=="1");
    txtPath.Text = ExtractFilePath(Document.Filename);
    txtPath.OnDblClick = "SelectFolder"
    txtPath.ShowHint = true;
    txtPath.Hint = "Enter path to where folder structure should be created. Doubleclick for Folder Dialog.";

    leftPos += txtPath.Width + 2;

    var spButton = new TSpeedButton(form);
    spButton.Parent = grpOptions;
    spButton.SetBounds( leftPos, topPos + 4*mul, 23*mul, 22*mul);
    spButton.Glyph.LoadFromFile(Script.Path + "/folder_explorer.bmp");
    spButton.Anchors = akTop + akRight;
    spButton.OnClick ="SelectFolderButton";

    topPos += vspace;
    leftPos = mleft;

    var lblProject = new TLabel(form);
    lblProject.Parent = grpOptions;
    lblProject.Alignment = taRightJustify;
    lblProject.Caption = "or Project:";
    lblProject.SetBounds(leftPos, topPos + 5*mul, 60*mul, 16*mul);
    lblProject.Anchors = akLeft + akTop;
    lblProject.ShowHint = true;
    lblProject.Hint = "Select Project where folder structure should be created.";

    leftPos += lblProject.Width + 2*mul;

    cmbProject = new TComboBox(form);
    cmbProject.Parent = grpOptions;
    cmbProject.Style = csOwnerDrawFixed; // If set to csDropDown (default value) then keyboard entry is possible
    cmbProject.ItemHeight = 16;
    cmbProject.Anchors = akLeft + akTop + akRight;
    cmbProject.SetBounds( leftPos, topPos + 2*mul, groupWidth - mleft*2 - lblProject.Width -4*mul , 24*mul);
    cmbProject.OnChange = "SelectProjectPath";
    cmbProject.Sorted = true;
    cmbProject.ShowHint = true;
    cmbProject.Hint = "Select Project where folder structure should be created.";
    UpdateProjects(cmbProject);

    topPos += vspace + mtop/2;
    leftPos = mleft;

    var lblLayout = new TLabel(form);
    lblLayout.Parent = grpOptions;
    lblLayout.Caption = "Structure:";
    lblLayout.SetBounds(leftPos, topPos + 5*mul, 60*mul, 16*mul);
    lblLayout.Anchors = akLeft + akTop;
    lblLayout.ShowHint = true;
    lblLayout.Hint = "Select Folder Structure Layout to use or enter name to create new Layout.";

    leftPos += lblLayout.Width + 2*mul;

    cmbLayout = new TComboBox(form);
    cmbLayout.Parent = grpOptions;
	cmbLayout.Style = csDropDown; // csOwnerDrawFixed, If set to csDropDown (default value) then keyboard entry is possible
	cmbLayout.ItemHeight = 16;
    cmbLayout.Anchors = akLeft + akTop + akRight;
    cmbLayout.SetBounds( leftPos, topPos + 2*mul, groupWidth - mleft*2 - lblLayout.Width -20*mul, 24*mul);
    cmbLayout.OnChange = "SelectLayout";
    cmbLayout.Sorted = true;
    cmbLayout.ShowHint = true;
    cmbLayout.Hint = "Select Folder Structure Layout to use or enter name to create new Layout.";
    UpdateLayouts(cmbLayout);
    cmbLayout.ItemIndex = settings.ReadInteger("Settings","current", 1);

    topPos += vspace + mtop;
    leftPos = mleft;

    var btnCreate = new TButton(form);
    btnCreate.Parent = grpOptions;
    btnCreate.Caption = "Create Folder Structure";
    btnCreate.Default = true;
    btnCreate.modalResult = mrOK;
    btnCreate.Anchors = akLeft + akTop + akRight;
    btnCreate.SetBounds(leftPos, topPos, grpOptions.Width - mleft*2, 30*mul);

    grpOptions.Height = topPos + mtop + mtop + 13*mul;

  	grpEdit = new TGroupBox(form);
	grpEdit.Parent = form;
    grpEdit.Caption = cmbLayout.Text;
    grpEdit.Anchors = akLeft + akTop + akRight + akBottom;
    grpEdit.Width = groupWidth - 18*mul; // Why do I have to subtract 18 from the width of the Groupbox???
    grpEdit.Height = form.Height - grpOptions.Height - margin*2 - 50*mul;
    grpEdit.Left = margin;
    grpEdit.Top = grpOptions.Height + margin + margin;

    topPos = mtop + mtop;
    leftPos = mleft;

    mmEdit = new TMemo(form);
    mmEdit.Parent = grpEdit;
    mmEdit.Enabled = false;
    mmEdit.SetBounds(leftPos, topPos, groupWidth - mleft*3, grpEdit.Height - 70*mul);
    mmEdit.Anchors = akLeft + akTop + akRight + akBottom;
    mmEdit.WordWrap = false;
    mmEdit.ScrollBars = ssBoth;
    mmEdit.ShowHint = true;
    mmEdit.Hint = "Enter your folder structure layout here. Lines ending with a slash/backslash is considered a folder.";

    topPos += mmEdit.Height + 8*mul;

    btnDelete = new TButton(form);
    btnDelete.Parent = grpEdit;
    btnDelete.Caption = "Delete Layout";
    btnDelete.modalResult = mrNone;
    btnDelete.OnClick = "DeleteLayout";
    btnDelete.Enabled = false;
    btnDelete.Anchors = akLeft + akBottom;
    btnDelete.SetBounds(leftPos, topPos, 95*mul, 25*mul);

    leftPos += groupWidth - mleft *3 - 95*mul;

    btnSave = new TButton(form);
    btnSave.Parent = grpEdit;
    btnSave.Caption = "Save Layout";
    btnSave.OnClick = "SaveLayout";
    btnSave.Enabled = false;
    btnSave.modalResult = mrNone;
    btnSave.Anchors = akRight + akBottom;
    btnSave.SetBounds(leftPos, topPos, 95*mul, 25*mul);

    SelectLayout();

    var res = form.Showmodal;
    if (res == mrOK) {
        CreateStructure();
    }

    delete form;
}

    /**
     * Helper function: Split string
     *
     * @param  string  str: The string to split
     * @param  string  splitChar: The character to split at
     * @return array
     */
function Split(str, splitChar) {
    var res = [str], count = 0, l = Length(splitChar), p = Pos(splitChar, str);
    while (p > 0) {
        res[count] = Copy(str, 0, p - 1);
        str = Copy(str,p + l,Length(str));
        count++;
        SetLength(res, count + 1);
        p = Pos(splitChar, str);
    }
    res[count] = str;
    return res;
}

/**
 * Helper function for calling SelectFolder with correct object
 *
 * @return void
 */
function SelectfolderButton() {
    SelectFolder(txtPath);
}

/**
 * Bring up dialog to select folder.
 *
 * @param  object   Sender
 *
 * @return void
 */
function SelectFolder(Sender) {
    var path = ExtractFilePath(Sender.Text);
    if (BrowseForFolder(path, Sender.Hint, true)) {
        Sender.Text = RegexReplace(path, "[\\r\\n]*$", "", true) + "\\";
    }
}

/**
 * Create file/folder structure
 *
 * @return void
 */
function CreateStructure() {

    var layoutName = cmbLayout.Text;
    if (Trim(layoutName) == "") {
        Alert("Select a layout first!");
        return;
    }
    var rootPath = ExtractFilePath(txtPath.Text);
    if  ((rootpath == "") || (rootpath == "\\") || (!DirectoryExists(rootPath))) {
        Alert("You must select a valid folder in the dialog");
        return;
    }

    var SL = new TStringList;
    SL.Assign(mmEdit.Lines);
    for (var i=0; i < SL.Count;i++) {
        if (SL[i] != "") {
            var sp = Split(SL[i], "<<<"); // Is ther a file to copy?
            var outputText = "";
            var fullPath = rootPath + RegexReplace(Replace(sp[0], "/","\\"), "^[\\\\]?(.*)", "$1", true);
            // Create folder
    		ExecuteCommand(Script.Path +  "createfolder.cmd \"" + ExtractFilePath(fullPath) + "\"", outputText);
            if (outputText != "") {
                Script.Message("Folder Error: " + outputText);
            }

            if (ExtractFileName(fullPath) != "") {
                outputText = "";
                if (Length(sp) == 2) {
                    // Copy file
            		ExecuteCommand(Script.Path + "copyfile.cmd \"" + sp[1] + "\" \"" + fullPath + "\"" , outputText);
                    // Stupid DOS copy command pipe error messages to stdout and not stderr
                    if ((outputText != "") && (RegexMatch(outputText,"\\d file\\(s\\) copied\\.",true) == ""))  {
                        Script.Message("File Error: " + outputText);
                    }
                }
                else {
                    // Create empty file
            		ExecuteCommand(Script.Path +  "createfile.cmd \"" + fullPath + "\"", outputText);
                    if (outputText != "") {
                        Script.Message("File Error: " + outputText);
                    }
                }
            }
        }
    }
    delete SL;
    settings.WriteInteger("Settings","current", cmbLayout.ItemIndex);

    Alert("Folder structure created in\n" + rootPath);
}

/**
 * Update Layout combobox
 *
 * @return void
 */
function UpdateLayouts(Sender) {
    var t = Sender.ItemIndex;
    layouts.ReadSections(Sender.Items);
    Sender.ItemIndex = t;
}

/**
 * Updates projects combobox
 *
 * @param  object   Sender
 *
 * @return void
 */
function UpdateProjects(Sender) {
    var t = Sender.ItemIndex;
    projects.ReadSections(Sender.Items);
    Sender.ItemIndex = t;
}


/**
 * Pass path from project to the path field
 *
 * @param  object   Sender
 *
 * @return void
 */
function SelectProjectPath(Sender) {
    var url = projects.ReadString(Sender.Text, "RootFolder", "");
    if (url != "") {
        txtPath.Text = url + "\\";
    }
}

/**
 * Delete layout
 *
 * @return void
 */
function DeleteLayout() {
    var layoutName = cmbLayout.Text;
    if (layouts.SectionExists(layoutName)) {
        layouts.EraseSection(layoutName);
        mmEdit.Lines.Text = "";
        cmbLayout.ItemIndex = -1;
        UpdateLayouts(cmbLayout);
        settings.WriteInteger("Settings","current", cmbLayout.ItemIndex);
        Alert("Layout \"" + layoutName + "\" removed");
    }
}

/**
 * Save layout
 *
 * @return void
 */
function SaveLayout() {
    if (mmEdit.Lines.Count == 0) {
        alert("Layout is blank! Nothing saved!");
        return;
    }
    var layoutName = cmbLayout.Text;
    var SL = new TStringList; // If I try storing the lines directly from the TMemo, I get an type error.
    SL.Assign(mmEdit.Lines);

    layouts.WriteInteger(layoutName, "count", SL.Count);
    for (var i=0; i < SL.Count;i++) {
        layouts.WriteString(layoutName, "line" + _t(i), SL[i]);
    }
    delete SL;

    UpdateLayouts(cmbLayout);
    settings.WriteInteger("Settings","current", cmbLayout.ItemIndex);
    Alert("Layout \"" + layoutName + "\" saved");
}

/**
 * Select layout and update TMemo
 *
 * @return void
 */
function SelectLayout() {
    var layoutName = cmbLayout.Text;
    if (layoutName == "") {
        mmEdit.Enabled = false;
        btnDelete.Enabled = false;
        btnSave.Enabled = false;
        return;
    }

    mmEdit.Enabled = true;
    btnDelete.Enabled = true;
    btnSave.Enabled = true;
    grpEdit.Caption = "Structure: " + layoutName;

    mmEdit.Lines.Text = "";

    if (layouts.SectionExists(layoutName)) {
        var count = layouts.ReadInteger(layoutName,"count", 0);
        for (var i=0; i < count;i++) {
            var line = layouts.ReadString(layoutName,"line" + _t(i), "");
            mmEdit.Lines.Add(line);
        }
    }
}

/**
 * Prevent modal window width from being resized below 200 pixels
 *
 * @param  object   Sender
 *
 * @return void
 */
function OnResize(Sender) {
    if (Sender.ClientWidth < 250) {
        Sender.ClientWidth = 250;
    }
    if (Sender.ClientHeight < 300) {
        Sender.ClientHeight = 300;
    }
}


/**
 * Cleanup when editor is closed
 *
 * @return void
 */
function OnExit() {
    if (projects)
        delete projects;
    if (layouts)
        delete layouts;
    if (settings)
        delete settings;
}

 /**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
  Alert("Create Folder Structure 1.1 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
Script.ConnectSignal("exit", "OnExit");

var bmp = new TBitmap;
LoadFileToBitmap(Script.Path + "folder_add.png", bmp);
var act = Script.RegisterDocumentAction("", "Create Folder Structure", "", "ShowModal");
Actions.SetIcon(act, bmp);
delete bmp;
